
#include "TFile.h"
#include "TTree.h"
#include "TBranch.h"
#include "TH1.h"
#include "TH2D.h"
#include "TAxis.h"
#include "TRandom.h"
#include "TFitResult.h"
#include "TFitResultPtr.h"
#include "TPad.h"
#include "TCanvas.h"

#include <iostream>
#include <vector>
#include <sstream>
#include <fstream>
#include <cmath>

#include "TArtSAMURAIParameters.hh"
#include "TArtNEBULAHPCPara.hh"
#include "TArtNEBULAPlaPara.hh"

#include "TArtAlias.hh"
#include "TArtHistManager.hh"

#include "TDataNEBULA.hh"

#include "TLinearFit.hh"

typedef TDataNEBULA data_t;

void nebula_tree_cosmic(const char* filename, int nentries = -1);

void nebula_tree_cosmic(const char* filename, int nentries)
{
  TFile* file = TFile::Open(filename);
  if(!file || file->IsZombie()) return;

  TTree* tree;
  file->GetObject("tree",tree);
  tree->Print();

  std::vector<Double_t>* id_v = new std::vector<Double_t>;
  std::vector<Double_t>* quraw_v = new std::vector<Double_t>;
  std::vector<Double_t>* qdraw_v = new std::vector<Double_t>;
  std::vector<Double_t>* turaw_v = new std::vector<Double_t>;
  std::vector<Double_t>* tdraw_v = new std::vector<Double_t>;
  TBranch *id_b, *quraw_b, *qdraw_b, *turaw_b, *tdraw_b;
  if(tree->SetBranchAddress("id",&id_v,&id_b) < 0) return;
  if(tree->SetBranchAddress("quraw",&quraw_v,&quraw_b) < 0) return;
  if(tree->SetBranchAddress("qdraw",&qdraw_v,&qdraw_b) < 0) return;
  if(tree->SetBranchAddress("turaw",&turaw_v,&turaw_b) < 0) return;
  if(tree->SetBranchAddress("tdraw",&tdraw_v,&tdraw_b) < 0) return;

  TH1* hidquped = new TH2D("hidquped", "id vs quped", 120, 0.5, 120.5, 300, 0, 2000);
  TH1* hidqdped = new TH2D("hidqdped", "id vs qdped", 120, 0.5, 120.5, 300, 0, 2000);
  TH1* hidqucal = new TH2D("hidqucal", "id vs qucal", 120, 0.5, 120.5, 300, 0, 70);
  TH1* hidqdcal = new TH2D("hidqdcal", "id vs qdcal", 120, 0.5, 120.5, 300, 0, 70);

  std::vector<TH1*> hlogqavepedzero(145,(TH1*)0);
  std::vector<TH1*> hlogqavepedzerocor(145,(TH1*)0);
  std::vector<TH1*> hivsinthetaqavecal(145,(TH1*)0);
  for(int i=0; i<1; ++i){
    {
      std::ostringstream name,title;
      name << "hlogqavepedzero" << i;
      title << "logqaveped vs zero ch" << i;
      hlogqavepedzero[i] = new TH2D(name.str().c_str(), title.str().c_str(), 200,3,8.3,200,-5,5);
    }
    {
      std::ostringstream name,title;
      name << "hlogqavepedzerocor" << i;
      title << "logqaveped vs zerocor ch" << i;
      hlogqavepedzerocor[i] = new TH2D(name.str().c_str(), title.str().c_str(), 200,3,8.3,200,-5,5);
    }
  }
  for(int i=0; i<145; ++i){
     {
       std::ostringstream name,title;
       name << "hivsinthetaqavecal" << i;
       title << "ivsintheta vs qavecal" << i;
       hivsinthetaqavecal[i] = new TH2D(name.str().c_str(), title.str().c_str(), 200, 0, 10, 300, 0, 150);
     }
   }

  TArtSAMURAIParameters *samuraiparameters = TArtSAMURAIParameters::Instance();
  samuraiparameters->LoadParameter((char*)"db/NEBULA.xml");

  std::vector<const TArtNEBULAPlaPara*> para_v;
  for(int i=0; i<144; ++i){
    int id=i+1;
    const TArtNEBULAPlaPara* para = samuraiparameters->FindNEBULAPlaPara(id);
    para_v.push_back(para);
  }

  if(nentries < 0) nentries = tree->GetEntriesFast();
  std::vector<data_t> data_v;
  for(int event=0; event<nentries; ++event) {
    if((event+1)%10000 == 0) std::cout << "\r converted events:" << event+1 << std::flush;

    Long64_t entry = tree->LoadTree(event);
    tree->GetEntry(entry);

    data_v.clear();
    for(int j=0; j<(int)id_v->size(); ++j){
      int id = (int)(*id_v)[j];
      double quraw = (*quraw_v)[j];
      double qdraw = (*qdraw_v)[j];
      double turaw = (*turaw_v)[j];
      double tdraw = (*tdraw_v)[j];
      const TArtNEBULAPlaPara* para = para_v[id-1];

      if(turaw < 0 || turaw > 4094 ||
	 tdraw < 0 || tdraw > 4094){
	continue;
      }

      if(para->GetSubLayer() == 0) continue;

      Double_t quped = quraw - para->GetQUPed();
      Double_t qdped = qdraw - para->GetQDPed();
      if(!(quped > 0 && qdped > 0)) continue;
      Double_t qucal = quped * para->GetQUCal();
      Double_t qdcal = qdped * para->GetQDCal();
      Double_t qaveped = sqrt(quped*qdped);
      Double_t qavecal = para->GetQAveCal()*sqrt(qucal*qdcal);

      Double_t tucal = turaw * para->GetTUCal() + para->GetTUOff();
      Double_t tdcal = tdraw * para->GetTDCal() + para->GetTDOff();
      Double_t tuslw = tucal;
      Double_t tdslw = tdcal;

      Double_t logquped = log(quped);
      Double_t logqdped = log(qdped);
      Double_t logquped2 = logquped*logquped;
      Double_t logqdped2 = logqdped*logqdped;
      Double_t logquped4 = logquped2*logquped2;
      Double_t logqdped4 = logqdped2*logqdped2;
      Double_t a = 0;
      Double_t b = para->GetTUSlwLog(0);
      Double_t c = para->GetTUSlwLog(1);
      Double_t d = para->GetTUSlwLog(2);
      Double_t e = para->GetTUSlwLog(3);
      Double_t f = para->GetTUSlwLog(4);

      tuslw -= (a +
		b*logquped +
		c*logquped2 +
		d*logquped2*logquped +
		e*logquped4 +
		f*logquped4*logquped);
      tdslw -= (a +
		b*logqdped +
		c*logqdped2 +
		d*logqdped2*logqdped +
		e*logqdped4 +
		f*logqdped4*logqdped);
      Double_t dtraw = tdraw - turaw;
      Double_t dtcal = tdcal - tucal;
      Double_t dtslw = tdslw - tuslw;
      //      Double_t taveraw = (turaw + tdraw)/2.;
      Double_t tavecal = (tucal + tdcal)/2 + para->GetTAveOff();
      Double_t taveslw = (tuslw + tdslw)/2 + para->GetTAveOff();
      Double_t poscal = dtcal * para->GetDTCal() + para->GetDTOff();
      Double_t posslw = dtslw * para->GetDTCal() + para->GetDTOff();

      std::vector<Double_t> pos(3);
      if(para->GetSubLayer() != 0){ // NEUT
	//	pos[0] = para->GetDetPos(0) + gRandom->Uniform(-6,6);
	pos[0] = para->GetDetPos(0);
      }else{ // VETO
	//	pos[0] = para->GetDetPos(0) + gRandom->Uniform(-16,16);
	pos[0] = para->GetDetPos(0);
      }
      pos[1] = posslw + para->GetDetPos(1);
      pos[2] = para->GetDetPos(2);

      qavecal /= 1+pos[1]*pos[1]*para->GetQAveCalAtt();

      data_t data;
      data.id = id;
      data.quraw = quraw;
      data.qdraw = qdraw;
      data.turaw = turaw;
      data.tdraw = tdraw;
      data.quped = quped;
      data.qdped = qdped;
      data.qucal = qucal;
      data.qdcal = qdcal;
      data.qaveped = qaveped;
      data.qavecal = qavecal;
      data.tucal = tucal;
      data.tdcal = tdcal;
      data.tuslw = tuslw;
      data.tdslw = tdslw;
      data.tavecal = tavecal;
      data.taveslw = taveslw;
      data.logquped = logquped;
      data.logqdped = logqdped;
      data.dtraw = dtraw;
      data.dtcal = dtcal;
      data.dtslw = dtslw;
      data.poscal = poscal;
      data.posslw = posslw;
      data.pos = pos;

      data_v.push_back(data);
    }

    static const int ntrackmin = 4;
    //    static const int ntrackmin = 3;
    //    static const int ntrackmin = 2;
    if((int)data_v.size() < ntrackmin) continue;

    // 今のalgorithmだとz方向にconstantな直線は計算できないのでcosmic rayに対してはzとyを入れ替えてしまう。
    std::vector<data_t> data_v_zyconvertion = data_v;
    for(int i=0; i<(int)data_v.size(); ++i){
      data_v_zyconvertion[i].pos[2] = data_v[i].pos[1];
      data_v_zyconvertion[i].pos[1] = data_v[i].pos[2];
    }

    double parFit[4];
    int last_size = data_v_zyconvertion.size();
    double amin;
    while(1){
      amin = OneTrackFit(data_v_zyconvertion, para_v, parFit);
      if(amin > 10){
	for(int i=0; i<(int)data_v_zyconvertion.size(); ++i){      
	  double t = data_v_zyconvertion[i].pos[2];
	  std::vector<double> pos = data_v_zyconvertion[i].pos;
	  double x,y,z;
	  TLinearFit::line(t, parFit, x, y, z);
	  double dx = pos[0]-x;
	  double dy = pos[1]-y;
	  double dz = pos[2]-z;
	  double dr = sqrt(dx*dx+dy*dy+dz*dz);
	  if(dr > 30){
	    data_v_zyconvertion.erase(data_v_zyconvertion.begin()+i);
	    data_v.erase(data_v.begin()+i);
	    --i;
	  }
	}
	if(last_size == (int)data_v_zyconvertion.size()) break;
	else last_size = data_v_zyconvertion.size();
      }else{
	break;
      }
      if((int)data_v.size() < ntrackmin) continue;
    }
    if((int)data_v.size() < ntrackmin) continue;
    if(amin > 10) continue;

    double costheta = 1/sqrt(parFit[1]*parFit[1]+parFit[3]*parFit[3]+1);
    double sintheta = sqrt(1-costheta*costheta);
    double cosphi = parFit[1]/sqrt(parFit[1]*parFit[1]+parFit[3]*parFit[3]);
    //    double sinphi = parFit[3]/sqrt(parFit[1]*parFit[1]+parFit[3]*parFit[3]);

    if(!(fabs(cosphi) > 0.99)) continue;

    double i_max = 0;
    double y_max = data_v[0].pos[1];
    double i_min = i_max;
    double y_min = y_max;
    for(int i=1; i<(int)data_v.size(); ++i){    
      double y = data_v[i].pos[1];
      if(y > y_max){
	y_max = y;
	i_max = i;
      }else if(y < y_min){
	y_min = y;
	i_min = i;
      }
    }

//	std::cout << atan(sqrt(dx*dx+dz*dz)/dy) << std::endl;
    for(int i=0; i<(int)data_v.size(); ++i){    
      double zero, zerocor;
      if(data_v[i].pos[1] - data_v[i_min].pos[1] > 
	 data_v[i_max].pos[1] - data_v[i].pos[1]){
	double dx = data_v[i].pos[0] - data_v[i_min].pos[0];
	double dy = data_v[i].pos[1] - data_v[i_min].pos[1];
	double dz = data_v[i].pos[2] - data_v[i_min].pos[2];
	double dr = sqrt(dx*dx+dy*dy+dz*dz);
	double dt = data_v[i_min].tavecal - data_v[i].tavecal;
	double dtcor = data_v[i_min].taveslw - data_v[i].taveslw;
	zero = -(dt - dr/vlight);
	zerocor = -(dtcor - dr/vlight);
      }else{
	double dx = data_v[i].pos[0] - data_v[i_max].pos[0];
	double dy = data_v[i].pos[1] - data_v[i_max].pos[1];
	double dz = data_v[i].pos[2] - data_v[i_max].pos[2];
	double dr = sqrt(dx*dx+dy*dy+dz*dz);
	double dt = data_v[i].tavecal - data_v[i_max].tavecal;
	double dtcor = data_v[i].taveslw - data_v[i_max].taveslw;
	zero = dt - dr/vlight;
	zerocor = dtcor - dr/vlight;
      }

      hlogqavepedzero[0]->Fill(log(data_v[i].qaveped),zero);
      hlogqavepedzerocor[0]->Fill(log(data_v[i].qaveped),zerocor);
    }

// take comment out to visualize.
// to finish visualization, need to ctrl+C.
//
//    visualize(data_v, para_v);
//    TLinearFit::DrawLine(parFit, kRed, -200, 400);
//     // yとz軸を入れ替えてvisualize
//     //    double parFit_zyconvertion[4];
//     //    std::cout << parFit[0] << " " << parFit[1] << " " << parFit[2] << " " << parFit[3] << std::endl;
//     //    parFit_zyconvertion[0] = parFit[0]-parFit[1]*parFit[2]/parFit[3];
//     //    parFit_zyconvertion[1] = parFit[1]/parFit[3];
//     //    parFit_zyconvertion[2] = -parFit[2]/parFit[3];
//     //    parFit_zyconvertion[3] = 1/parFit[3];
//     //    TLinearFit::DrawLine(parFit_zyconvertion, kRed, -200, 400);
//    gPad->GetCanvas()->Update();
//    TLinearFit::EventLoop();

    for(int i=0; i<(int)data_v.size(); ++i){
      data_t data = data_v[i];
      int id = data.id;
      //      const TArtNEBULAPlaPara* para = para_v[id-1];

      double quped = data.quped;
      double qdped = data.qdped;
      double qucal = data.qucal;
      double qdcal = data.qdcal;
      double qavecal = data.qavecal;

      hidquped->Fill(id, quped);
      hidqdped->Fill(id, qdped);
      hidqucal->Fill(id, qucal);
      hidqdcal->Fill(id, qdcal);

      hivsinthetaqavecal[0]->Fill(1/sintheta, qavecal);
      hivsinthetaqavecal[id]->Fill(1/sintheta, qavecal);      
    }
  }
  std::cout << std::endl;

  tree->ResetBranchAddresses();
}

